@@ -15,8 +15,6 @@
 #include <gtk/gtk.h>
 
 #include <dbus/dbus.h>
-#include <mce/mode-names.h>
-#include <mce/dbus-names.h>
 #include <hildon/hildon-program.h>
 #include <hildon/hildon-window.h>
 #include <hildon/hildon-button.h>
@@ -24,9 +22,11 @@
 #include <hildon/hildon-color-chooser.h>
 #include <hildon/hildon-color-chooser-dialog.h>
 #include <hildon/hildon-color-button.h>
+#include <libosso.h>
 #include <libintl.h>
 #define _(String) gettext (String)
 
+#include "xournal.h"
 #include "xo-callbacks.h"
 #include "xo-hildon_ui.h"
 #include "xo-support.h"
@@ -34,8 +34,32 @@
 extern char *theme_name;
 extern GtkUIManager *ui_manager;
 extern GtkWidget *winMain;
+extern GtkWidget *hildon_cur_toolbar_landscape,
+	*hildon_cur_toolbar_portrait,
+	*switch_to_toolbar;
+extern GtkWidget *hildon_toolbars[HILDON_NUM_TOOLBARS];
+extern GtkWidget *hildon_undo[HILDON_NUM_TOOLBARS];
+extern GtkWidget *hildon_redo[HILDON_NUM_TOOLBARS];
+
+gboolean hildon_cur_toolbar_visible = TRUE;
+
+char *hildonToolbars[HILDON_NUM_TOOLBARS] = {
+	"HildonToolBarNoteLandscape",
+	"HildonToolBarNotePortrait",
+	"HildonToolBarPdfLandscape",
+	"HildonToolBarPdfPortrait",
+	"HildonToolBarDrawLandscape",
+	"HildonToolBarDrawPortrait",
+	"HildonToolBarHandLandscape",
+	"HildonToolBarHandPortrait",
+	"HildonToolBarPenLandscape",
+	"HildonToolBarPenPortrait",
+	"HildonToolBarPenColorLandscape",
+	"HildonToolBarPenColorPortrait"
+};
 
 GtkWidget *tools_view, *settings_view, *journal_view;
+//extern gint two_half_button_click, hildon_button_pressed, hildon_button_released, hildon_timer_running_already;
 
 #define GLADE_HOOKUP_OBJECT(component,widget,name) \
   g_object_set_data_full (G_OBJECT (component), name, \
@@ -57,6 +81,10 @@ register_hildon_stock_icons (void)
 
   for (i=0;i<n_stock_icons;i++) {
 	  char *filename=calloc(1024,1);
+	  GtkIconSize *sizes = NULL;
+	  gint nsize;
+	  gint j;
+
 	  icon_set = gtk_icon_set_new ();
 	  icon_source = gtk_icon_source_new ();
 	  snprintf (filename, 1024, hildon_stock_icons[i].filename, theme_name);
@@ -72,45 +100,319 @@ register_hildon_stock_icons (void)
   g_object_unref (icon_factory);
 }
 
-static GtkWidget *add_menu_item (HildonAppMenu *menu, const gchar *title,
-		GCallback callback, gpointer user_data)
+static GtkWidget *add_item (HildonAppMenu *menu, const gchar *title,
+		GCallback callback, gint show, gpointer user_data, GtkWidget *button, gint filter)
 {
-	GtkWidget *button = hildon_button_new_with_text (
+	static GtkWidget *first = NULL;
+
+	if (filter) {
+		if (first != NULL) {
+			button = (button ? button : hildon_gtk_radio_button_new_from_widget (
+					HILDON_SIZE_AUTO,
+					GTK_RADIO_BUTTON (first)));
+		} else {
+			button = (button ? button: hildon_gtk_radio_button_new (
+				HILDON_SIZE_AUTO,
+				NULL));
+			first = button;
+		}
+
+		gtk_button_set_label (GTK_BUTTON (button), title);
+	} else {
+		button = (button ? button : hildon_button_new_with_text (
 			HILDON_SIZE_AUTO,
 			HILDON_BUTTON_ARRANGEMENT_VERTICAL,
 			title,
-			NULL);
-	gtk_widget_show (button);
+			NULL));
+	}
+
 	g_signal_connect_after (button, "clicked",
 			G_CALLBACK (callback),
 			user_data);
-	hildon_app_menu_append (menu, GTK_BUTTON(button));
+		
+	if (show) {
+		gtk_widget_show (button);
+	} else {
+		gtk_widget_hide (button);
+	}
+
+	if (filter) {
+		hildon_app_menu_add_filter (menu, GTK_BUTTON(button));
+		gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (button), FALSE);
+	} else {
+		hildon_app_menu_append (menu, GTK_BUTTON(button));
+	}
 
 	return button;
 }
 
+void add_menu (HildonAppMenu *menu, XournalHildonMenu *menu_items, int num_items, gpointer user_data) {
+	int i=0;
+
+	for (i=0; i<num_items;i++) {
+		menu_items[i].widget = add_item (menu,
+				menu_items[i].title,
+				menu_items[i].callback,
+				menu_items[i].show,
+				user_data,
+			       	menu_items[i].widget,
+				0);
+	}
+}
+
+void add_filter (HildonAppMenu *menu, XournalHildonMenu *menu_items, int num_items, gpointer user_data) {
+	int i=0;
+
+	for (i=0; i<num_items;i++) {
+		menu_items[i].widget = add_item (menu,
+				menu_items[i].title,
+				menu_items[i].callback,
+				menu_items[i].show,
+				user_data, 
+				menu_items[i].widget,
+				1);
+	}
+}
+
 HildonAppMenu *hildon_create_menu (gpointer user_data)
 {
-	int i=0;
 	HildonAppMenu *app_menu = HILDON_APP_MENU(hildon_app_menu_new ());
 
-	int num_item = sizeof (fremantle_menu_entries) /
-		sizeof (fremantle_menu_entries[0]);
-
-	for (i=0; i<num_item;i++) {
-		add_menu_item (app_menu, fremantle_menu_entries[i].title,
-				fremantle_menu_entries[i].callback, user_data);
-	}
+	add_menu (app_menu, fremantle_menu_file_entries, 
+		sizeof (fremantle_menu_file_entries)/sizeof(XournalHildonMenu),
+		user_data);
+	add_menu (app_menu, fremantle_menu_tools_entries,
+		sizeof (fremantle_menu_tools_entries)/sizeof(XournalHildonMenu),
+		user_data);
+	add_filter (app_menu, fremantle_menu_filters,
+		sizeof (fremantle_menu_filters)/sizeof(XournalHildonMenu),
+		user_data);
 
 	return app_menu;
 }
 
+void hildon_pen_activate (GtkWidget *item, gpointer user_data)
+{
+}
+
+void hildon_eraser_activate (GtkWidget *item, gpointer user_data)
+{
+}
+
+void hildon_highlighter_activate (GtkWidget *item, gpointer user_data)
+{
+}
+
+void hildon_zoom_activate (GtkWidget *item, gpointer user_data)
+{
+}
+
+void hildon_layer_activate (GtkWidget *item, gpointer user_data)
+{
+}
+
+void hildon_page_activate (GtkWidget *item, gpointer user_data)
+{
+}
+
+void hildon_paper_activate (GtkWidget *item, gpointer user_data)
+{
+}
+
+void hildon_toolbar_refresh_color (GtkWidget *item, gpointer user_data)
+{
+	gint toolbar = (gint) user_data;
+	gchar widget_path[200];
+	GtkWidget *widget;
+
+	switch (ui.cur_brush->color_no) {
+		case COLOR_BLACK:
+    			sprintf (widget_path, "/%s/Black", hildonToolbars[toolbar]);
+			widget = gtk_ui_manager_get_widget (ui_manager, widget_path);
+			break;
+		case COLOR_BLUE:
+    			sprintf (widget_path, "/%s/Blue", hildonToolbars[toolbar]);
+			widget = gtk_ui_manager_get_widget (ui_manager, widget_path);
+			break;
+		case COLOR_RED:
+    			sprintf (widget_path, "/%s/Red", hildonToolbars[toolbar]);
+			widget = gtk_ui_manager_get_widget (ui_manager, widget_path);
+			break;
+		case COLOR_GREEN:
+    			sprintf (widget_path, "/%s/Green", hildonToolbars[toolbar]);
+			widget = gtk_ui_manager_get_widget (ui_manager, widget_path);
+			break;
+		case COLOR_YELLOW:
+    			sprintf (widget_path, "/%s/Yellow", hildonToolbars[toolbar]);
+			widget = gtk_ui_manager_get_widget (ui_manager, widget_path);
+			break;
+		case COLOR_ORANGE:
+    			sprintf (widget_path, "/%s/Orange", hildonToolbars[toolbar]);
+			widget = gtk_ui_manager_get_widget (ui_manager, widget_path);
+			break;
+		case COLOR_MAGENTA:
+    			sprintf (widget_path, "/%s/Fuchsia", hildonToolbars[toolbar]);
+			widget = gtk_ui_manager_get_widget (ui_manager, widget_path);
+			break;
+		default:
+			widget = NULL;
+	}
+
+	if (widget) {
+		gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON(widget), TRUE);
+	}
+}
+
+void hildon_toolbar_switch_do (gint toolbar_no)
+{
+  if (switch_to_toolbar_landscape != NULL) {
+    return;
+  }
+
+  switch_to_toolbar_portrait = hildon_cur_toolbar_portrait;
+  switch_to_toolbar_landscape = hildon_cur_toolbar_landscape;
+
+  if (hildon_cur_toolbar_visible) {
+    if (device_is_portrait_mode ()) {
+      gtk_widget_hide (GTK_WIDGET(hildon_cur_toolbar_portrait));
+      gtk_widget_show (GTK_WIDGET(hildon_toolbars[toolbar_no+1]));
+    } else {
+      gtk_widget_hide (GTK_WIDGET(hildon_cur_toolbar_landscape));
+      gtk_widget_show (GTK_WIDGET(hildon_toolbars[toolbar_no]));
+    }
+  }
+
+  hildon_cur_toolbar_landscape = hildon_toolbars[toolbar_no];
+  hildon_cur_toolbar_portrait = hildon_toolbars[toolbar_no+1];
+}
+
+void hildon_toolbar_switch_back ()
+{
+  if (switch_to_toolbar_landscape == NULL) {
+    return;
+  }
+
+  if (hildon_cur_toolbar_visible) {
+    if (device_is_portrait_mode ()) {
+      gtk_widget_hide (GTK_WIDGET(hildon_cur_toolbar_portrait));
+      gtk_widget_show (GTK_WIDGET(switch_to_toolbar_portrait));
+    } else {
+      gtk_widget_hide (GTK_WIDGET(hildon_cur_toolbar_landscape));
+      gtk_widget_show (GTK_WIDGET(switch_to_toolbar_landscape));
+    }
+  }
+
+  hildon_cur_toolbar_landscape = switch_to_toolbar_landscape;
+  hildon_cur_toolbar_portrait = switch_to_toolbar_portrait;
+
+  switch_to_toolbar_landscape = NULL;
+  switch_to_toolbar_portrait = NULL;
+}
+
+void hildon_toolbar_switch (GtkWidget *item, gpointer user_data)
+{
+  gint toolbar_no = (gint) user_data;
+
+  if (gtk_toggle_tool_button_get_active (GTK_TOGGLE_TOOL_BUTTON(item))) {
+    hildon_toolbar_switch_do (toolbar_no);
+  } else {
+    hildon_toolbar_switch_back ();
+  }
+}
+
+void hildon_draw_switch (GtkWidget *item, gpointer user_data)
+{
+  hildon_toolbar_switch_back ();
+
+  if (device_is_portrait_mode ()) {
+     gtk_widget_hide (GTK_WIDGET(hildon_cur_toolbar_portrait));
+     gtk_widget_show (GTK_WIDGET(hildon_toolbars[HILDON_TOOLBAR_DRAW_PORTRAIT]));
+  } else {
+     gtk_widget_hide (GTK_WIDGET(hildon_cur_toolbar_landscape));
+     gtk_widget_show (GTK_WIDGET(hildon_toolbars[HILDON_TOOLBAR_DRAW_LANDSCAPE]));
+  }
+
+  hildon_cur_toolbar_portrait = hildon_toolbars[HILDON_TOOLBAR_DRAW_PORTRAIT];
+  hildon_cur_toolbar_landscape = hildon_toolbars[HILDON_TOOLBAR_DRAW_LANDSCAPE];
+
+  hildon_cur_toolbar_visible = TRUE;
+}
+
+void hildon_pdf_switch (GtkWidget *item, gpointer user_data)
+{
+  hildon_toolbar_switch_back ();
+
+  if (device_is_portrait_mode ()) {
+     gtk_widget_hide (GTK_WIDGET(hildon_cur_toolbar_portrait));
+     gtk_widget_show (GTK_WIDGET(hildon_toolbars[HILDON_TOOLBAR_PDF_PORTRAIT]));
+  } else {
+     gtk_widget_hide (GTK_WIDGET(hildon_cur_toolbar_landscape));
+     gtk_widget_show (GTK_WIDGET(hildon_toolbars[HILDON_TOOLBAR_PDF_LANDSCAPE]));
+  }
+
+  hildon_cur_toolbar_portrait = hildon_toolbars[HILDON_TOOLBAR_PDF_PORTRAIT];
+  hildon_cur_toolbar_landscape = hildon_toolbars[HILDON_TOOLBAR_PDF_LANDSCAPE];
+
+  hildon_cur_toolbar_visible = TRUE;
+}
+
+void hildon_note_switch (GtkWidget *item, gpointer user_data)
+{
+  hildon_toolbar_switch_back ();
+
+  if (device_is_portrait_mode ()) {
+     gtk_widget_hide (GTK_WIDGET(hildon_cur_toolbar_portrait));
+     gtk_widget_show (GTK_WIDGET(hildon_toolbars[HILDON_TOOLBAR_NOTE_PORTRAIT]));
+  } else {
+     gtk_widget_hide (GTK_WIDGET(hildon_cur_toolbar_landscape));
+     gtk_widget_show (GTK_WIDGET(hildon_toolbars[HILDON_TOOLBAR_NOTE_LANDSCAPE]));
+  }
+
+  hildon_cur_toolbar_portrait = hildon_toolbars[HILDON_TOOLBAR_NOTE_PORTRAIT];
+  hildon_cur_toolbar_landscape = hildon_toolbars[HILDON_TOOLBAR_NOTE_LANDSCAPE];
+
+  hildon_cur_toolbar_visible = TRUE;
+}
+
+void hildon_file_menu_activate (GtkWidget *item, gpointer user_data)
+{
+	int i,num_items;
+
+	num_items = sizeof (fremantle_menu_tools_entries)/
+			sizeof (XournalHildonMenu);
+	for (i=0;i<num_items;i++) {
+		gtk_widget_hide (GTK_WIDGET(fremantle_menu_tools_entries[i].widget));
+	}
+
+	num_items = sizeof (fremantle_menu_file_entries)/
+			sizeof (XournalHildonMenu);
+	for (i=0;i<num_items;i++) {
+		gtk_widget_show (GTK_WIDGET(fremantle_menu_file_entries[i].widget));
+	}
+}
+
+void hildon_tools_menu_activate (GtkWidget *item, gpointer user_data)
+{
+	int i,num_items;
+
+	num_items = sizeof (fremantle_menu_file_entries)/
+			sizeof (XournalHildonMenu);
+	for (i=0;i<num_items;i++) {
+		gtk_widget_hide (GTK_WIDGET(fremantle_menu_file_entries[i].widget));
+	}
+
+	num_items = sizeof (fremantle_menu_tools_entries)/
+			sizeof (XournalHildonMenu);
+	for (i=0;i<num_items;i++) {
+		gtk_widget_show (GTK_WIDGET(fremantle_menu_tools_entries[i].widget));
+	}
+}
+
 GtkWidget*
-create_winMain (void)
+create_winMain (osso_context_t *ctx)
 {
   HildonProgram *hildon_program;
   HildonWindow *hildon_window;
-  GtkWidget *hildon_toolbar;
 #ifdef MAEMO_4
   GtkWidget *hildon_mainmenu;
 #else
@@ -118,12 +420,19 @@ create_winMain (void)
 #endif
   GtkWidget *vboxMain;
   GtkWidget *scrolledwindowMain;
+  GtkWidget *hildonSelector;
   GtkAccelGroup *accel_group;
   GtkActionGroup *action_group;
   GtkAction *action_eraser, *action_pen;
   GtkWidget *widget_eraser, *widget_pen;
+  GtkWidget *undo_widget;
+  GtkWidget *redo_widget;
+  GtkWidget *widget;
+  GtkToolItem *colorItem;
   GError *error;
+  gchar *label;
   char xml_path[2048];
+  int i;
 
   register_hildon_stock_icons ();
 
@@ -141,28 +450,41 @@ create_winMain (void)
 
   gtk_action_group_add_actions (action_group, hildon_menu_entries,
 		  G_N_ELEMENTS (hildon_menu_entries), hildon_window);
+
   gtk_action_group_add_actions (action_group, hildon_toggle_entries,
 		  G_N_ELEMENTS (hildon_toggle_entries), hildon_window);
+
   gtk_action_group_add_radio_actions (action_group, hildon_view_radio_entries,
 		  G_N_ELEMENTS(hildon_view_radio_entries), 0, NULL, hildon_window);
+
   gtk_action_group_add_radio_actions (action_group, hildon_paperColor_radio_entries,
 		  G_N_ELEMENTS(hildon_paperColor_radio_entries), 0, NULL, hildon_window);
+
   gtk_action_group_add_radio_actions (action_group, hildon_paperStyle_radio_entries,
 		  G_N_ELEMENTS(hildon_paperStyle_radio_entries), 1, NULL, hildon_window);
+
   gtk_action_group_add_radio_actions (action_group, hildon_tools_radio_entries,
 		  G_N_ELEMENTS(hildon_tools_radio_entries), 0, G_CALLBACK(hildon_tools_radio_action), hildon_window);
+
   gtk_action_group_add_toggle_actions (action_group, hildon_toolsOther_toggle_entries,
 		  G_N_ELEMENTS(hildon_toolsOther_toggle_entries), hildon_window);
+
   gtk_action_group_add_radio_actions (action_group, hildon_penOptions_radio_entries,
-		  G_N_ELEMENTS(hildon_penOptions_radio_entries), 2, NULL, hildon_window);
+		  G_N_ELEMENTS(hildon_penOptions_radio_entries), 2, G_CALLBACK(hildon_penOptions_radio_action), hildon_window);
+
+  gtk_action_group_add_radio_actions (action_group, hildon_penColor_radio_entries,
+		  G_N_ELEMENTS(hildon_penColor_radio_entries), 3, G_CALLBACK(hildon_penColor_radio_action), hildon_window);
+
   gtk_action_group_add_radio_actions (action_group,
 		  hildon_eraserThicknessOptions_radio_entries,
 		  G_N_ELEMENTS(hildon_eraserThicknessOptions_radio_entries), 1, NULL,
 		  hildon_window);
+
   gtk_action_group_add_radio_actions (action_group,
 		  hildon_eraserTypeOptions_radio_entries,
 		  G_N_ELEMENTS(hildon_eraserTypeOptions_radio_entries), 0, NULL,
 		  hildon_window);
+
   gtk_action_group_add_radio_actions (action_group,
 		  hildon_highlighterThicknessOptions_radio_entries,
 		  G_N_ELEMENTS(hildon_highlighterThicknessOptions_radio_entries), 1, NULL,
@@ -200,6 +522,114 @@ create_winMain (void)
   GLADE_HOOKUP_OBJECT (GTK_WIDGET(hildon_window),
 	 scrolledwindowMain, "scrolledwindowMain");
 
+//  gtk_widget_show_all (GTK_WIDGET (hildon_mainmenu));
+  gtk_widget_show_all (GTK_WIDGET (hildon_window));
+
+  /* Load toolbars */
+  hildon_toolbars[HILDON_TOOLBAR_NOTE_PORTRAIT] = gtk_ui_manager_get_widget (ui_manager, "/HildonToolBarNotePortrait");
+  hildon_window_add_toolbar (hildon_window, GTK_TOOLBAR (hildon_toolbars[HILDON_TOOLBAR_NOTE_PORTRAIT]));
+  hildon_toolbars[HILDON_TOOLBAR_NOTE_LANDSCAPE] = gtk_ui_manager_get_widget (ui_manager, "/HildonToolBarNoteLandscape");
+  hildon_window_add_toolbar (hildon_window, GTK_TOOLBAR (hildon_toolbars[HILDON_TOOLBAR_NOTE_LANDSCAPE]));
+
+  hildon_toolbars[HILDON_TOOLBAR_PDF_PORTRAIT] = gtk_ui_manager_get_widget (ui_manager, "/HildonToolBarPdfPortrait");
+  hildon_window_add_toolbar (hildon_window, GTK_TOOLBAR (hildon_toolbars[HILDON_TOOLBAR_PDF_PORTRAIT]));
+  hildon_toolbars[HILDON_TOOLBAR_PDF_LANDSCAPE] = gtk_ui_manager_get_widget (ui_manager, "/HildonToolBarPdfLandscape");
+  hildon_window_add_toolbar (hildon_window, GTK_TOOLBAR (hildon_toolbars[HILDON_TOOLBAR_PDF_LANDSCAPE]));
+
+  hildon_toolbars[HILDON_TOOLBAR_DRAW_PORTRAIT] = gtk_ui_manager_get_widget (ui_manager, "/HildonToolBarDrawPortrait");
+  hildon_window_add_toolbar (hildon_window, GTK_TOOLBAR (hildon_toolbars[HILDON_TOOLBAR_DRAW_PORTRAIT]));
+  hildon_toolbars[HILDON_TOOLBAR_DRAW_LANDSCAPE] = gtk_ui_manager_get_widget (ui_manager, "/HildonToolBarDrawLandscape");
+  hildon_window_add_toolbar (hildon_window, GTK_TOOLBAR (hildon_toolbars[HILDON_TOOLBAR_DRAW_LANDSCAPE]));
+
+  hildon_toolbars[HILDON_TOOLBAR_HAND_PORTRAIT] = gtk_ui_manager_get_widget (ui_manager, "/HildonToolBarHandPortrait");
+  hildon_window_add_toolbar (hildon_window, GTK_TOOLBAR (hildon_toolbars[HILDON_TOOLBAR_HAND_PORTRAIT]));
+  hildon_toolbars[HILDON_TOOLBAR_HAND_LANDSCAPE] = gtk_ui_manager_get_widget (ui_manager, "/HildonToolBarHandLandscape");
+  hildon_window_add_toolbar (hildon_window, GTK_TOOLBAR (hildon_toolbars[HILDON_TOOLBAR_HAND_LANDSCAPE]));
+
+  hildon_toolbars[HILDON_TOOLBAR_PEN_PORTRAIT] = gtk_ui_manager_get_widget (ui_manager, "/HildonToolBarPenPortrait");
+  hildon_window_add_toolbar (hildon_window, GTK_TOOLBAR (hildon_toolbars[HILDON_TOOLBAR_PEN_PORTRAIT]));
+  hildon_toolbars[HILDON_TOOLBAR_PEN_LANDSCAPE] = gtk_ui_manager_get_widget (ui_manager, "/HildonToolBarPenLandscape");
+  hildon_window_add_toolbar (hildon_window, GTK_TOOLBAR (hildon_toolbars[HILDON_TOOLBAR_PEN_LANDSCAPE]));
+
+  hildon_toolbars[HILDON_TOOLBAR_COLOR_PORTRAIT] = gtk_ui_manager_get_widget (ui_manager, "/HildonToolBarPenColorPortrait");
+  hildon_window_add_toolbar (hildon_window, GTK_TOOLBAR (hildon_toolbars[HILDON_TOOLBAR_COLOR_PORTRAIT]));
+  hildon_toolbars[HILDON_TOOLBAR_COLOR_LANDSCAPE] = gtk_ui_manager_get_widget (ui_manager, "/HildonToolBarPenColorLandscape");
+  hildon_window_add_toolbar (hildon_window, GTK_TOOLBAR (hildon_toolbars[HILDON_TOOLBAR_COLOR_LANDSCAPE]));
+
+  for (i=0;i<HILDON_NUM_TOOLBARS;i++) {
+    char widget_path[256];
+
+    sprintf (widget_path, "/%s/Hand", hildonToolbars[i]);
+
+    widget = gtk_ui_manager_get_widget (ui_manager, widget_path);
+    g_signal_connect (G_OBJECT (widget), "clicked",
+	G_CALLBACK (hildon_toolbar_switch), (gpointer)HILDON_TOOLBAR_HAND_LANDSCAPE);
+
+    sprintf (widget_path, "/%s/Pen", hildonToolbars[i]);
+
+    widget = gtk_ui_manager_get_widget (ui_manager, widget_path);
+    g_signal_connect (G_OBJECT (widget), "clicked",
+	G_CALLBACK (hildon_toolbar_switch), (gpointer)HILDON_TOOLBAR_PEN_LANDSCAPE);
+
+    sprintf (widget_path, "/%s/PenColor", hildonToolbars[i]);
+
+    widget = gtk_ui_manager_get_widget (ui_manager, widget_path);
+    g_signal_connect (G_OBJECT (widget), "clicked",
+	G_CALLBACK (hildon_toolbar_switch), (gpointer)HILDON_TOOLBAR_COLOR_LANDSCAPE);
+  }
+
+  g_signal_connect (G_OBJECT(hildon_toolbars[HILDON_TOOLBAR_COLOR_PORTRAIT]), "show",
+	G_CALLBACK (hildon_toolbar_refresh_color), (gpointer)HILDON_TOOLBAR_COLOR_PORTRAIT);
+  g_signal_connect (G_OBJECT(hildon_toolbars[HILDON_TOOLBAR_COLOR_LANDSCAPE]), "show",
+	G_CALLBACK (hildon_toolbar_refresh_color), (gpointer)HILDON_TOOLBAR_COLOR_LANDSCAPE);
+
+  // Undo / Redo buttons
+  undo_widget = gtk_ui_manager_get_widget (ui_manager, "/HildonToolBarNoteLandscape/Undo");
+  redo_widget = gtk_ui_manager_get_widget (ui_manager, "/HildonToolBarNoteLandscape/Redo");
+
+  hildon_undo[HILDON_TOOLBAR_NOTE_LANDSCAPE] = undo_widget;
+  hildon_redo[HILDON_TOOLBAR_NOTE_LANDSCAPE] = redo_widget;
+
+  hildon_undo[HILDON_TOOLBAR_PDF_LANDSCAPE] = undo_widget;
+  hildon_redo[HILDON_TOOLBAR_PDF_LANDSCAPE] = redo_widget;
+
+  hildon_undo[HILDON_TOOLBAR_DRAW_LANDSCAPE] = undo_widget;
+  hildon_redo[HILDON_TOOLBAR_DRAW_LANDSCAPE] = redo_widget;
+
+  pickerButton = hildon_picker_button_new (HILDON_SIZE_AUTO,
+		  HILDON_BUTTON_ARRANGEMENT_VERTICAL);
+  hildonSelector = hildon_touch_selector_new_text ();
+
+  hildon_button_set_title (HILDON_BUTTON(pickerButton), _("Select layer"));
+
+  hildon_touch_selector_set_column_selection_mode (
+		HILDON_TOUCH_SELECTOR(hildonSelector),
+ 		HILDON_TOUCH_SELECTOR_SELECTION_MODE_SINGLE);
+
+  g_signal_connect (G_OBJECT (hildonSelector), "changed",
+		G_CALLBACK (on_comboLayer_changed), (gpointer) "maemo");
+
+  /* Set a default [not yet configurable] toolbar */
+  hildon_cur_toolbar_landscape = hildon_toolbars[HILDON_TOOLBAR_NOTE_LANDSCAPE];
+  hildon_cur_toolbar_portrait = hildon_toolbars[HILDON_TOOLBAR_NOTE_PORTRAIT];
+
+  /* Hide all */
+  for (i=0;i<HILDON_NUM_TOOLBARS;i++) {
+	gtk_widget_hide_all (GTK_WIDGET (hildon_toolbars[i]));
+  }
+
+  hildon_picker_button_set_selector (HILDON_PICKER_BUTTON(pickerButton),
+		HILDON_TOUCH_SELECTOR(hildonSelector));
+
+  hildon_app_menu_append (hildon_mainmenu, GTK_BUTTON(pickerButton));
+  gtk_widget_show_all (pickerButton);
+
+  fremantle_menu_file_entries[HILDON_SHOW_LAYER_POS].widget = GTK_WIDGET (pickerButton);
+
+  g_signal_connect ((gpointer) hildon_window, "delete_event",
+		G_CALLBACK (on_winMain_delete_event),
+		NULL);
+
 #ifdef MAEMO_4
   hildon_mainmenu = gtk_ui_manager_get_widget (ui_manager, "/HildonMainMenu");
   hildon_window_set_menu (hildon_window, GTK_MENU (hildon_mainmenu));
@@ -208,22 +638,6 @@ create_winMain (void)
   hildon_window_set_app_menu (hildon_window, hildon_mainmenu);
 #endif
 
-  hildon_toolbar = gtk_ui_manager_get_widget (ui_manager, "/HildonToolBar");
-  hildon_window_add_toolbar (hildon_window, GTK_TOOLBAR (hildon_toolbar));
-  
-  gtk_widget_show_all (GTK_WIDGET (hildon_mainmenu));
-  gtk_widget_show_all (GTK_WIDGET (hildon_toolbar));
-  gtk_widget_show_all (GTK_WIDGET (hildon_window));
-
-  hildon_toolbar = gtk_ui_manager_get_widget (ui_manager, "/HildonToolBarPortrait");
-  hildon_window_add_toolbar (hildon_window, GTK_TOOLBAR (hildon_toolbar));
-  gtk_widget_hide_all (GTK_WIDGET (hildon_toolbar));
-
-  g_signal_connect ((gpointer) hildon_window, "delete_event",
-		G_CALLBACK (on_winMain_delete_event),
-		NULL);
-
-
   return GTK_WIDGET (hildon_window);
 }
 
@@ -376,7 +790,7 @@ create_aboutDialog (void)
   gtk_box_pack_start (GTK_BOX (dialog_vbox2), labelTitle, FALSE, FALSE, 3);
   gtk_label_set_justify (GTK_LABEL (labelTitle), GTK_JUSTIFY_CENTER);
 
-  labelInfo = gtk_label_new (_("Written by Denis Auroux\nPorted to Maemo by Aniello Del Sorbo\n       http://xournal.sourceforge.net/       "));
+  labelInfo = gtk_label_new (_("Written by Denis Auroux       http://xournal.sourceforge.net/       "));
   gtk_widget_show (labelInfo);
   gtk_box_pack_start (GTK_BOX (dialog_vbox2), labelInfo, FALSE, FALSE, 0);
   gtk_label_set_justify (GTK_LABEL (labelInfo), GTK_JUSTIFY_CENTER);
@@ -593,26 +1007,21 @@ hildon_create_tools_view (void)
 	gtk_widget_show (toolbarPen);
 	gtk_box_pack_start (GTK_BOX (vboxMain), toolbarPen, FALSE, FALSE, 0);
 
-	printf ("creating Fine\n");
 	buttonFine = (GtkWidget*) gtk_radio_tool_button_new (NULL);
 	gtk_tool_button_set_label (GTK_TOOL_BUTTON (buttonFine), _("Fine"));
-	printf ("creating pixmap\n");
-	tmp_image = create_pixmap (win, "thin.png");
+	tmp_image = create_pixmap (GTK_WIDGET(win), "thin.png");
 	gtk_widget_show (tmp_image);
 	gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (buttonFine), tmp_image);
 	gtk_widget_show (buttonFine);
 	gtk_tool_item_set_homogeneous (GTK_TOOL_ITEM (buttonFine), FALSE);
-	printf ("adding to box\n");
 	gtk_container_add (GTK_CONTAINER (toolbarPen), buttonFine);
-	printf ("added to box\n");
 //	gtk_widget_set_size_request (buttonFine, 24, -1);
 	gtk_radio_tool_button_set_group (GTK_RADIO_TOOL_BUTTON (buttonFine), buttonFine_group);
 	buttonFine_group = gtk_radio_tool_button_get_group (GTK_RADIO_TOOL_BUTTON (buttonFine));
 
-	printf ("creating Medium\n");
 	buttonMedium = (GtkWidget*) gtk_radio_tool_button_new (NULL);
 	gtk_tool_button_set_label (GTK_TOOL_BUTTON (buttonMedium), _("Medium"));
-	tmp_image = create_pixmap (win, "medium.png");
+	tmp_image = create_pixmap (GTK_WIDGET(win), "medium.png");
 	gtk_widget_show (tmp_image);
 	gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (buttonMedium), tmp_image);
 	gtk_widget_show (buttonMedium);
@@ -622,10 +1031,9 @@ hildon_create_tools_view (void)
 	gtk_radio_tool_button_set_group (GTK_RADIO_TOOL_BUTTON (buttonMedium), buttonFine_group);
 	buttonFine_group = gtk_radio_tool_button_get_group (GTK_RADIO_TOOL_BUTTON (buttonMedium));
 
-	printf ("creating Thick\n");
 	buttonThick = (GtkWidget*) gtk_radio_tool_button_new (NULL);
 	gtk_tool_button_set_label (GTK_TOOL_BUTTON (buttonThick), _("Thick"));
-	tmp_image = create_pixmap (win, "thick.png");
+	tmp_image = create_pixmap (GTK_WIDGET(win), "thick.png");
 	gtk_widget_show (tmp_image);
 	gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (buttonThick), tmp_image);
 	gtk_widget_show (buttonThick);
@@ -685,7 +1093,7 @@ hildon_create_settings_view (void)
 }
 
 void
-hildon_grab_volume_keys (unsigned long val)
+hildon_grab_volume_keys (int val)
 {
 	Atom atom;
 	GdkDisplay *display = NULL;
@@ -693,7 +1101,7 @@ hildon_grab_volume_keys (unsigned long val)
 	display = gdk_drawable_get_display (winMain->window);
 	atom = gdk_x11_get_xatom_by_name_for_display (display, "_HILDON_ZOOM_KEY_ATOM");
 	XChangeProperty (GDK_DISPLAY_XDISPLAY (display),
-			GDK_WINDOW_XID(winMain), atom, XA_INTEGER, 32,
+			GDK_WINDOW_XID(winMain->window), atom, XA_INTEGER, 32,
 			PropModeReplace, (unsigned char *) &val, 1);
 }
 
@@ -701,50 +1109,65 @@ void
 hildon_rotate_ui (const gchar *mode)
 {
 	GtkWidget *hildon_toolbar;
-	printf("rotate: %s\n", mode);
 
-	if (!strcmp (mode, MCE_ORIENTATION_PORTRAIT)) {
-		// hide landscape toolbar
-		hildon_toolbar = gtk_ui_manager_get_widget (ui_manager, "/HildonToolBar");
-		gtk_widget_hide (hildon_toolbar);
-		// rotate
-		hildon_gtk_window_set_portrait_flags(GTK_WINDOW(winMain),
-				HILDON_PORTRAIT_MODE_REQUEST);
-		// show portrait toolbar
-		hildon_toolbar = gtk_ui_manager_get_widget (ui_manager, "/HildonToolBarPortrait");
-		gtk_widget_show (hildon_toolbar);
-	}
-
-	if (!strcmp (mode, MCE_ORIENTATION_LANDSCAPE)) {
-		// hide landscape toolbar
-		hildon_toolbar = gtk_ui_manager_get_widget (ui_manager, "/HildonToolBarPortrait");
-		gtk_widget_hide (hildon_toolbar);
-		// rotate
-		hildon_gtk_window_set_portrait_flags(GTK_WINDOW(winMain),
-				~HILDON_PORTRAIT_MODE_REQUEST);
-		// show portrait toolbar
-		hildon_toolbar = gtk_ui_manager_get_widget (ui_manager, "/HildonToolBar");
-		gtk_widget_show (hildon_toolbar);
-	}
-}
-
-DBusHandlerResult
-mce_filter_func (DBusConnection *connection, DBusMessage *message, void *data)
-{
-	DBusMessageIter iter;
-	const gchar *mode = NULL;
-
-	printf ("MCE\n");
-
-	if (dbus_message_is_signal(message, MCE_SIGNAL_IF, MCE_DEVICE_ORIENTATION_SIG)) {
-		printf ("is a orientation signal\n");
-		if (dbus_message_iter_init(message, &iter)) {
-			printf ("getting mode\n");
-			dbus_message_iter_get_basic(&iter, &mode);
-
-			hildon_rotate_ui (mode);
+	if (!strcmp (mode, "portrait")) {
+		if (hildon_cur_toolbar_visible) {
+			// hide landscape toolbar
+			gtk_widget_hide (hildon_cur_toolbar_landscape);
+			// show portrait toolbar
+			gtk_widget_show (hildon_cur_toolbar_portrait);
 		}
 	}
 
-	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+	if (!strcmp (mode, "landscape")) {
+		if (hildon_cur_toolbar_visible) {
+			// hide portrait toolbar
+			gtk_widget_hide (hildon_cur_toolbar_portrait);
+			// show landscape toolbar
+			gtk_widget_show (hildon_cur_toolbar_landscape);
+		}
+	}
 }
+
+void
+hildon_orientationChanged_process (GdkScreen *screen, GtkWidget *widget)
+{
+	if (device_is_portrait_mode ()) {
+		hildon_rotate_ui ("portrait");
+	} else {
+		hildon_rotate_ui ("landscape");
+	}
+}
+
+gboolean
+device_is_portrait_mode()
+{
+	GdkScreen *screen = gtk_widget_get_screen (winMain);
+	int width = gdk_screen_get_width (screen);
+	int height = gdk_screen_get_height (screen);
+
+	if (width > height) {
+		return FALSE;
+	} else {
+		return TRUE;
+	}
+}
+
+/*
+gint
+hildon_button_pressed_timedout (gpointer data) {
+	GdkEventButton *event = (GdkEventButton *)data;
+
+	if (two_half_button_click) {
+		return;
+	}
+
+	hildon_button_pressed = 0;
+	hildon_button_released = 0;
+	hildon_timer_running_already = 0;
+
+	printf ("Timer expired\n");
+
+	return 0;
+}
+*/
